# Copyright (C) 2009 Chris Larson <clarson@kergoth.com>
# Released under the MIT license (see COPYING.MIT for the terms)
#
# gitver.bbclass provides a GITVER variable which is a (fairly) sane version,
# for use in ${PV}, extracted from the ${S} git checkout, assuming it is one.

def git_drop_tag_prefix(version):
    import re
    if re.match("v\d", version):
        return version[1:]
    else:
        return version

# Assume that the Git repository is in ${S} unless the recipe tells us otherwise
GITVER_S ?= "${S}"

GIT_TAGADJUST = "git_drop_tag_prefix(version)"
GITVER = "${@get_git_pv('${GITVER_S}', d, match=None, tagadjust=lambda version:${GIT_TAGADJUST})}"
GITSHA = "${@get_git_hash('${GITVER_S}', d)}"

TAG_MACHINE = "${BASE_MACHINE}"
# If we're building the tests, get the tag from the underlying machine instead
TAG_MACHINE_cheetah-tests = "cheetah"
TAG_MACHINE_panther-tests = "panther"
TAG_MACHINE_qemux86 = "panther"
TAG_MACHINE_panther-x11 = "panther"
TAG_MACHINE_cheetah-x11 = "cheetah"

BS_GITVER = "${@get_git_pv('${GITVER_S}', d, match='release/${TAG_MACHINE}/*', tagadjust=brightsign_tagadjust)}"

def brightsign_tagadjust(version):
    import re
    return re.sub(r'^([^/]*/)*', '', version)

def libmedia_tagadjust(version):
    import re
    matches = re.match('^.*/(\d+\.\d+|\d+\.\d+\.\d+|\d+\.\d+\.\d+\.\d+)([^.]+)?$', version)
    if matches:
        return matches.group(1)
    else:
        bb.fatal("libmedia_tagadjust: unknown format: " + version)

def get_git_hash2(path, git_env, d):
    return bb.process.run(["/usr/bin/git", "rev-parse", "--short", "HEAD"], cwd=path, env=git_env, shell=False)[0].rstrip()

def get_git_hash(path, d):
    # There's no reason to run git under pseudo - it will only slow us
    # down and perhaps doing so is causing git to erroneously believe
    # that the tree is dirty sometimes.
    git_env = os.environ.copy()
    git_env["PSEUDO_DISABLED"] = "1"
    git_env["PSEUDO_UNLOAD"] = "1"
    return get_git_hash2(path, git_env, d)

# This checks to see if there are any files in the working
# tree or the index that are different to HEAD. If they are
# then we tack +local on the end of the version to indicate
# that there are local changes. This stops us matching
# anything from the shared sstate cache. We don't make PV
# change based on the mtime of the most recently changed file
# because that would cause us to fetch the source again every
# time which can often be avoided during development. (It
# turns out that "git describe" has this functionality built
# in in via its --dirty flag so we only use this if "git
# describe" fails.)
#
# To avoid triggering bug #17361 setting BS_GITVER_NEVER_MODIFIED
# disables this behaviour.

def get_git_dirty(rootdir, git_env):
    try:
        never_modified = d.getVar('BS_GITVER_NEVER_MODIFIED', True)
        if never_modified and never_modified == "1":
            return ""
        args = [ "(/usr/bin/git diff --quiet && /usr/bin/git diff --cached --quiet) || echo '+local'" ]
        maybe_dirty = bb.process.run(args, cwd=rootdir, env=git_env, shell=True)[0].rstrip()
        if maybe_dirty != "" and maybe_dirty != "+local":
            bb.fatal("git failure in gitver_notag: " + maybe_dirty)
            maybe_dirty = "+error"
        return maybe_dirty
    except Exception, exc:
        # Don't fail in here during recipe parsing
        return "+local-unknown"

def get_git_pv(path, d, match=None, tagadjust=None):
    import os
    import bb.process

    need_machine = d.getVar('COMPATIBLE_MACHINE', True)
    if need_machine:
        import re
        compat_machines = (d.getVar('MACHINEOVERRIDES', True) or "").split(":")
        for m in compat_machines:
            if re.match(need_machine, m):
                break
        else:
            return "0.0"

    # There's no reason to run git under pseudo - it will only slow us
    # down and perhaps doing so is causing git to erroneously believe
    # that the tree is dirty sometimes.
    git_env = os.environ.copy()
    git_env["PSEUDO_DISABLED"] = "1"
    git_env["PSEUDO_UNLOAD"] = "1"

    never_modified = d.getVar('BS_GITVER_NEVER_MODIFIED', True)
    if never_modified and never_modified == "1":
        mark_if_dirty = False
    else:
        mark_if_dirty = True

    rootdir = os.path.abspath(d.getVar("GITVER_S", True))
    try:
        args = [ "/usr/bin/git", "describe", "--tags", "--first-parent" ]
        if mark_if_dirty:
            args.extend(["--dirty=+local"])
        if match:
            args.extend(["--match", match])

        ver = bb.process.run(args, cwd=rootdir, env=git_env, shell=False)[0].rstrip()
    except Exception, exc:
        # Don't fail here during recipe parsing
        bb.warn("get_git_pv %s(%s): %s" % (d.getVar("PN", True), rootdir, exc))
        ver = "0.0-not-in-git"

    if not ver:
        try:
            ver = get_git_hash2(rootdir, git_env, d)
            if mark_if_dirty:
                ver += get_git_dirty(rootdir, git_env)
        except Exception, exc:
            bb.fatal(str(exc))

        if ver:
            return "0.0+%s" % ver
        else:
            return "0.0"
    else:
        if tagadjust:
            ver = tagadjust(ver)
        return ver

__BB_DONT_CACHE = "1"
